<?php

namespace SoosyzeCore\System\Services;

use Soosyze\Components\Util\Util;

class Modules
{
    /**
     * @var \SoosyzeCore\QueryBuilder\Services\Query
     */
    protected $query;

    protected $translate;

    public function __construct($query, $translate)
    {
        $this->query     = $query;
        $this->transalte = $translate;
    }

    /**
     * Si le module est installé.
     *
     * @param string $title Nom du module.
     *
     * @return array
     */
    public function has($title)
    {
        return $this->query
                ->from('module_active')
                ->where('title', $title)
                ->fetch();
    }

    /**
     * Si le module est requis par le module virtuel "core".
     *
     * @param string $title Nom du module.
     *
     * @return array
     */
    public function isRequiredCore($title)
    {
        return $this->query
                ->from('module_require')
                ->where('title_module', $title)
                ->where('title_required', 'Core')
                ->lists('title_required');
    }

    /**
     * Si le module est requis par un autre module installé.
     *
     * @param string $title Nom du module.
     *
     * @return array
     */
    public function isRequiredForModule($title)
    {
        $output = $this->query
            ->from('module_active')
            ->leftJoin('module_require', 'title', 'module_require.title_required')
            ->where('title', $title)
            ->isNotNull('title_module')
            ->lists('title_module');

        return array_unique($output);
    }

    public function listModuleActive(array $columns = [])
    {
        $modules = $this->query
            ->select($columns)
            ->from('module_active')
            ->fetchAll();

        $out = [];
        foreach ($modules as $value) {
            $out[ $value[ 'title' ] ] = $value;
        }

        return $out;
    }

    public function listModuleActiveNotRequire(array $columns = [])
    {
        return $this->query
                ->select($columns)
                ->from('module_active')
                ->leftJoin('module_require', 'title', 'module_require.title_required')
                ->isNull('title_module')
                ->lists('title');
    }

    /**
     * Désinstalle un module.
     *
     * @param string $title Nom du module.
     */
    public function uninstallModule($title)
    {
        $this->query
            ->from('module_active')
            ->delete()
            ->where('title', $title)
            ->execute();

        $this->query
            ->from('module_require')
            ->delete()
            ->where('title_module', $title)
            ->execute();

        $this->query
            ->from('module_controller')
            ->delete()
            ->where('title', $title)
            ->execute();
    }

    /**
     * Créer un module à partir du contenu de son fichier composer.json
     *
     * @param array $composer Données du module.
     */
    public function create(array $composer)
    {
        $module = $composer[ 'extra' ][ 'soosyze' ];
        /* Enregistrement du module. */
        $this->query
            ->insertInto('module_active', [ 'title', 'version' ])
            ->values([ $module[ 'title' ], $composer[ 'version' ] ])
            ->execute();

        /* Enregistrement des contrôleurs. */
        $this->query
            ->insertInto('module_controller', [ 'title', 'controller' ])
            ->values([ $module[ 'title' ], $module[ 'controller' ] ])
            ->execute();

        if (isset($module[ 'require' ])) {
            /* Enregistrement des dépendances. */
            $this->query->insertInto('module_require', [
                'title_module', 'title_required', 'version'
            ]);

            foreach ($module[ 'require' ] as $require => $version) {
                $this->query->values([ $module[ 'title' ], $require, $version ]);
            }

            $this->query->execute();
        }
    }

    /**
     * Installe les fichiers de traduction.
     *
     * @param array $modules  Liste des noms de modules à installer
     * @param array $composer Liste de tous les fichiers composer
     * @param bool  $crushed  Si la mise à jour de la traduction ne prend pas en compte existante.
     */
    public function loadTranslations(array $modules, array $composer, $crushed = false)
    {
        $path         = $this->transalte->getPath();

        $strTranslations = [];

        /* Réuni tous les fichiers de traductions par langues */
        foreach ($modules as $title) {
            foreach ($composer[ $title ][ 'translations' ] as $lang => $translations) {
                foreach ($translations as $translation) {
                    if (isset($strTranslations[ $lang ])) {
                        $strTranslations[ $lang ] += Util::getJson($translation);
                    } else {
                        $strTranslations[ $lang ] = Util::getJson($translation);
                    }
                }
            }
        }

        /* Enregistre les fichiers de traductions */
        foreach ($strTranslations as $lang => $translation) {
            if (file_exists("$path/$lang.json")) {
                $current = $crushed
                    ? Util::getJson("$path/$lang.json")
                    : [];

                Util::saveJson($path, $lang, $current + $translation);
            } else {
                Util::createJson($path, $lang, $translation);
            }
        }
    }
}
